@@ -1,6 +1,6 @@
 /*-------------------------------------------------------------------------
 |   rxtx is a native interface to serial ports in java.
-|   Copyright 1997, 1998, 1999 by Trent Jarvi trentjarvi@yahoo.com
+|   Copyright 1997-2002 by Trent Jarvi taj@www.linux.org.uk
 |
 |   This library is free software; you can redistribute it and/or
 |   modify it under the terms of the GNU Library General Public
@@ -16,35 +16,36 @@
 |   License along with this library; if not, write to the Free
 |   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 --------------------------------------------------------------------------*/
-package  javax.comm;
+package  gnu.io;
 
-import  gnu.io.*;
-import  java.io.*;
-import  java.util.*;
-import  javax.comm.*;
+import  java.io.FileDescriptor;
+import  java.util.Vector;
+import  java.util.Enumeration;
 
+/**
+* @author Trent Jarvi
+* @version %I%, %G%
+* @since JDK1.0
+*/
 
-
-/*------------------------------------------------------------------------------
-Lots of stubs here.  Taken from the javadoc material produced from Sun's
-commapi porting file.  Not used yet.
-------------------------------------------------------------------------------*/
-
-public class CommPortIdentifier 
+public class CommPortIdentifier extends Object /* extends Vector? */
 {
-	public static final int PORT_SERIAL = 1;  // Serial Port
-	public static final int PORT_PARALLEL = 2; // Parallel Port
+	public static final int PORT_SERIAL   = 1;  // rs232 Port
+	public static final int PORT_PARALLEL = 2;  // Parallel Port
+	public static final int PORT_I2C      = 3;  // i2c Port
+	public static final int PORT_RS485    = 4;  // rs485 Port
+	public static final int PORT_RAW      = 5;  // Raw Port
 	private String PortName;
-	private boolean Available;    
+	private boolean Available = true;    
 	private String Owner;    
 	private CommPort commport;
 	private CommDriver RXTXDriver;
- 	private static CommPortIdentifier   CommPortIndex;
-	private CommPortIdentifier next;
+ 	static CommPortIdentifier   CommPortIndex;
+	CommPortIdentifier next;
 	private int PortType;
-	private static boolean debug=true;
-	private static Object Sync;
-	private static String Properties;
+	private final static boolean debug = false;
+	static Object Sync;
+	Vector ownershipListener;
 
 
 
@@ -61,7 +62,6 @@ public class CommPortIdentifier
 	{
 		if(debug) System.out.println("CommPortIdentifier:static initialization()");
 		Sync = new Object();
-		Properties=System.getProperty("java.home")+"//lib//javax.com.properties";
 		try 
 		{
 			CommDriver RXTXDriver = (CommDriver) Class.forName("gnu.io.RXTXCommDriver").newInstance();
@@ -69,16 +69,26 @@ public class CommPortIdentifier
 		} 
 		catch (Throwable e) 
 		{
-			System.err.println(e + "thrown while loading " + "gnu.io.RXTXCommDriver");
+			System.err.println(e + " thrown while loading " + "gnu.io.RXTXCommDriver");
 		}
+
+		String OS;
+
+		OS = System.getProperty("os.name");
+		if(OS.toLowerCase().indexOf("linux") == -1)
+		{
+			if (debug)
+				System.out.println("Have not implemented native_psmisc_report_owner(PortName)); in CommPortIdentifier");
+		}
+		System.loadLibrary( "rxtxSerial" );
 	}
 	CommPortIdentifier ( String pn, CommPort cp, int pt, CommDriver driver) 
 	{
-		this.PortName	= pn;
-		this.commport	= cp;
-		this.PortType	= pt;
-		this.next	= null;
-		this.RXTXDriver	= driver;
+		PortName        = pn;
+		commport        = cp;
+		PortType        = pt;
+		next            = null;
+		RXTXDriver      = driver;
 
 	}
 
@@ -91,26 +101,18 @@ public class CommPortIdentifier
 	exceptions:     none.
 	comments:
 ------------------------------------------------------------------------------*/
-	public static void addPortName(String s, int type, RXTXCommDriver c) 
+	public static void addPortName(String s, int type, CommDriver c) 
 	{ 
 
-		if(debug) System.out.println("CommPortIdentifier:addPortName(" + s + ")");
-		if(debug) System.out.println("getting Security Manager");
-		SecurityManager MySecurity = System.getSecurityManager();
-		if (MySecurity != null) 
-		{
-			if(debug) System.out.println("Got Security Manager");
-			MySecurity.checkRead(Properties);
-		}
-		if(debug) System.out.println("Adding to List");
+		if(debug) System.out.println("CommPortIdentifier:addPortName("+s+")");
 		AddIdentifierToList(new CommPortIdentifier(s, null, type, c));
 	}
 /*------------------------------------------------------------------------------
 	AddIdentifierToList()
-	accept:         
+	accept:        The cpi to add to the list. 
 	perform:        
 	return: 	
-	exceptions:     
+	exceptions:    
 	comments:
 ------------------------------------------------------------------------------*/
 	private static void AddIdentifierToList( CommPortIdentifier cpi)
@@ -121,6 +123,7 @@ public class CommPortIdentifier
 			if (CommPortIndex == null) 
 			{
 				CommPortIndex = cpi;
+				if(debug) System.out.println("CommPortIdentifier:AddIdentifierToList() null");
 			}
 			else
 			{ 
@@ -128,6 +131,7 @@ public class CommPortIdentifier
 				while (index.next != null)
 				{
 					index = index.next;
+					if(debug) System.out.println("CommPortIdentifier:AddIdentifierToList() index.next");
 				}
 				index.next = cpi;
 			} 
@@ -139,11 +143,25 @@ public class CommPortIdentifier
 	perform:
 	return:
 	exceptions:
-	comments:
+	comments:   
 ------------------------------------------------------------------------------*/
 	public void addPortOwnershipListener(CommPortOwnershipListener c) 
 	{ 
 		if(debug) System.out.println("CommPortIdentifier:addPortOwnershipListener()");
+
+		/*  is the Vector instantiated? */
+
+		if( ownershipListener == null )
+		{
+			ownershipListener = new Vector();
+		}
+
+		/* is the ownership listener already in the list? */
+
+		if ( ownershipListener.contains(c) == false)
+		{
+			ownershipListener.addElement(c);
+		}
 	}
 /*------------------------------------------------------------------------------
 	getCurrentOwner()
@@ -151,13 +169,12 @@ public class CommPortIdentifier
 	perform:
 	return:
 	exceptions:
-	comments:
+	comments:    
 ------------------------------------------------------------------------------*/
 	public String getCurrentOwner() 
 	{ 
 		if(debug) System.out.println("CommPortIdentifier:getCurrentOwner()");
-		String s=new String();
-		return(s);
+		return( Owner );
 	}
 /*------------------------------------------------------------------------------
 	getName()
@@ -170,8 +187,7 @@ public class CommPortIdentifier
 	public String getName() 
 	{ 
 		if(debug) System.out.println("CommPortIdentifier:getName()");
-		String s = new String();
-		return(s);
+		return( PortName );
 	}
 /*------------------------------------------------------------------------------
 	getPortIdentifier()
@@ -179,29 +195,28 @@ public class CommPortIdentifier
 	perform:
 	return:
 	exceptions:
-	comments:
+	comments:   
 ------------------------------------------------------------------------------*/
 	static public CommPortIdentifier getPortIdentifier(String s) throws NoSuchPortException 
 	{ 
 		if(debug) System.out.println("CommPortIdentifier:getPortIdentifier(" + s +")");
-		System.out.println("configure --enable-RXTXIDENT is for developers only");
-		SecurityManager MySecurity = System.getSecurityManager();
-		if (MySecurity != null) { MySecurity.checkRead(Properties); }
 		CommPortIdentifier index = CommPortIndex;
 
 		synchronized (Sync) 
 		{
 			while (index != null) 
 			{
-				if (index.PortName.equals(s))
-					break;
+				if (index.PortName.equals(s)) break;
 				index = index.next;
 			}
 		}
-		if (index != null) 
-			return index;
-		else 
+		if (index != null) return index;
+		else
+		{
+			if ( debug )
+				System.out.println("not found!" + s);
 			throw new NoSuchPortException();
+		}
 	}
 /*------------------------------------------------------------------------------
 	getPortIdentifier()
@@ -209,12 +224,23 @@ public class CommPortIdentifier
 	perform:
 	return:
 	exceptions:
-	comments:
+	comments:    
 ------------------------------------------------------------------------------*/
-	static public CommPortIdentifier getPortIdentifier(CommPort c) throws NoSuchPortException 	
+	static public CommPortIdentifier getPortIdentifier(CommPort p) 
+		throws NoSuchPortException 	
 	{ 
-		if(debug) System.out.println("CommPortIdentifier:getPortIdentifier()");
-		System.out.println("configure --enable-RXTXIDENT is for developers only");
+		if(debug) System.out.println("CommPortIdentifier:getPortIdentifier(CommPort)");
+		CommPortIdentifier c = CommPortIndex;
+		synchronized( Sync )
+		{
+			while ( c != null && c.commport != p )
+				c = c.next;
+		}
+		if ( c != null )
+			return (c);
+
+		if ( debug )
+			System.out.println("not found!" + p.getName());
 		throw new NoSuchPortException();
 	}
 /*------------------------------------------------------------------------------
@@ -227,7 +253,7 @@ public class CommPortIdentifier
 ------------------------------------------------------------------------------*/
 	static public Enumeration getPortIdentifiers() 
 	{ 
-		if(debug) System.out.println("CommPortIdentifier:getPortIdentifiers()");
+		if(debug) System.out.println("static CommPortIdentifier:getPortIdentifiers()");
 		return new CommPortEnumerator();
 	}
 /*------------------------------------------------------------------------------
@@ -241,7 +267,7 @@ public class CommPortIdentifier
 	public int getPortType() 
 	{ 
 		if(debug) System.out.println("CommPortIdentifier:getPortType()");
-		return(1);
+		return( PortType );
 	}
 /*------------------------------------------------------------------------------
 	isCurrentlyOwned()
@@ -249,12 +275,12 @@ public class CommPortIdentifier
 	perform:
 	return:
 	exceptions:
-	comments:
+	comments:    
 ------------------------------------------------------------------------------*/
-	public boolean isCurrentlyOwned() 
+	public synchronized boolean isCurrentlyOwned() 
 	{ 
 		if(debug) System.out.println("CommPortIdentifier:isCurrentlyOwned()");
-		return(false);
+		return(!Available);
 	}
 /*------------------------------------------------------------------------------
 	open()
@@ -264,11 +290,13 @@ public class CommPortIdentifier
 	exceptions:
 	comments:
 ------------------------------------------------------------------------------*/
-	public CommPort open(FileDescriptor f) throws UnsupportedCommOperationException 
+	public synchronized CommPort open(FileDescriptor f) throws UnsupportedCommOperationException 
 	{ 
-		if(debug) System.out.println("CommPortIdentifier:open()");
+		if(debug) System.out.println("CommPortIdentifier:open(FileDescriptor)");
 		throw new UnsupportedCommOperationException();
 	}
+	private native String native_psmisc_report_owner(String PortName);
+
 /*------------------------------------------------------------------------------
 	open()
 	accept:      application makeing the call and milliseconds to block
@@ -280,22 +308,42 @@ public class CommPortIdentifier
 ------------------------------------------------------------------------------*/
 	private boolean HideOwnerEvents;
 
-	public synchronized CommPort open(String TheOwner, int i) throws PortInUseException 
+	public synchronized CommPort open(String TheOwner, int i) 
+		throws gnu.io.PortInUseException 
 	{ 
-		commport = RXTXDriver.getCommPort(PortName,PortType);
-		if(debug) System.out.println("CommPortIdentifier:open()");
-		if(Available)
+		if(debug) System.out.println("CommPortIdentifier:open("+TheOwner + ", " +i+")");
+		if (Available == false)
+		{
+			synchronized (Sync)
+			{
+				fireOwnershipEvent(CommPortOwnershipListener.PORT_OWNERSHIP_REQUESTED);
+				try
+				{
+					wait(i);
+				}
+				catch ( InterruptedException e ) { }
+			}
+		}
+		if (Available == false)
+		{
+			throw new gnu.io.PortInUseException(getCurrentOwner());
+		}
+		if(commport == null)
+		{
+			commport = RXTXDriver.getCommPort(PortName,PortType);
+		}
+		if(commport != null)
 		{
-			Available = false;
 			Owner = TheOwner;
+			Available=false;
+			fireOwnershipEvent(CommPortOwnershipListener.PORT_OWNED);
 			return commport;
 		}
-		/* 
-		possibly talk other jvms about getting the port?
-		NativeFindOwner(PortName);
-		throw new PortInUseException(); 
-		*/
-		return null;
+		else
+		{
+			throw new gnu.io.PortInUseException(
+					native_psmisc_report_owner(PortName));
+		}
 	}
 /*------------------------------------------------------------------------------
 	removePortOwnership()
@@ -308,38 +356,48 @@ public class CommPortIdentifier
 	public void removePortOwnershipListener(CommPortOwnershipListener c) 
 	{ 
 		if(debug) System.out.println("CommPortIdentifier:removePortOwnershipListener()");
+		/* why is this called twice? */
+		if(ownershipListener != null)
+			ownershipListener.removeElement(c);
 	}
 
-}
-
-class CommPortEnumerator implements Enumeration {
-	private static boolean debug=true;
-	private CommPortIdentifier CPI;
-
 /*------------------------------------------------------------------------------
-        nextElement()
-        accept:
-        perform:
-        return:
-        exceptions:
-        comments:
+	internalClosePort()
+	accept:       None
+	perform:      clean up the Ownership information and send the event
+	return:       None
+	exceptions:   None
+	comments:     None
 ------------------------------------------------------------------------------*/
-	public Object nextElement() 
-	{ 
-		if(debug) System.out.println("CommPortEnumerator:nextElement()");
-		return(CPI);
+	synchronized void internalClosePort() 
+	{
+		if(debug) System.out.println("CommPortIdentifier:internalClosePort()");
+		Owner = null;
+		Available = true;
+		commport = null;
+		/*  this tosses null pointer?? */
+		notifyAll();
+		fireOwnershipEvent(CommPortOwnershipListener.PORT_UNOWNED);
 	}
 /*------------------------------------------------------------------------------
-        hasMoreElements()
-        accept:
-        perform:
-        return:
-        exceptions:
-        comments:
+	fireOwnershipEvent()
+	accept:
+	perform:
+	return:
+	exceptions:
+	comments:
 ------------------------------------------------------------------------------*/
-	public boolean hasMoreElements() 
-	{ 
-		if(debug) System.out.println("CommPortEnumerator:hasMoreElements");
-		return(false); 
+	void fireOwnershipEvent(int eventType)
+	{
+		if(debug) System.out.println("CommPortIdentifier:fireOwnershipEvent( " + eventType + " )");
+		if (ownershipListener != null)
+		{
+			CommPortOwnershipListener c;
+			for ( Enumeration e = ownershipListener.elements();
+				e.hasMoreElements(); 
+				c.ownershipChange(eventType))
+				c = (CommPortOwnershipListener) e.nextElement();
+		}
 	}
 }
+
