@@ -1,6 +1,6 @@
 /*-------------------------------------------------------------------------
 |   rxtx is a native interface to serial ports in java.
-|   Copyright 1997, 1998, 1999 by Trent Jarvi trentjarvi@yahoo.com
+|   Copyright 1997-2002 by Trent Jarvi taj@www.linux.org.uk
 |
 |   This library is free software; you can redistribute it and/or
 |   modify it under the terms of the GNU Library General Public
@@ -16,24 +16,26 @@
 |   License along with this library; if not, write to the Free
 |   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 --------------------------------------------------------------------------*/
-/* 
+/*
    fear he who enter here.  It appears that things have changed.  An attempt
    has been made to put things the way the should be.
 
-	basic problem is LP_P* appears to be ifdefed __KERNEL__ in the
-        lp.h header file on linux.  It also looks as if LP_P* should be
-        LP_* for POSOX compliance.  So... Some P's got chopped out.
-        Its not clear what LP_PACK is supposed to become so its commented out
-        below.
+        It compiles and ParallelBlackBox runs.  No further guarantees.
+
+	Well... One.. it will print "Hello World!" on an epson DX 10 printer.
+	you know.. the 10 character per second daisy wheel printer ;)
 
-        Some errors may have occured during the change.  It compiles and
-        ParallelBlackBox runs.  No further garantees.
-   
         - Trent Jarvi
 */
 
+#if defined(__MWERKS__)/* dima */
+#include "LPRPort.h"
+#else /* dima */
 #include "config.h"
+/* work around for libc5 */
+/*#include <typedefs_md.h>*/
 #include "gnu_io_LPRPort.h"
+#endif /* dima */
 #include <time.h>
 #include <unistd.h>
 #include <stdio.h>
@@ -44,6 +46,8 @@
 #include <sys/errno.h>
 #include <sys/param.h>
 #include <sys/time.h>
+#include <stdlib.h>
+
 #ifdef HAVE_TERMIOS_H
 #	include <termios.h>
 #endif
@@ -60,49 +64,56 @@
 #	include <linux/lp.h>
 #endif
 #if defined(__FreeBSD__)
-#	include <machine/lpt.h>
+/* #	include <machine/lpt.h> ? is this changed or wrong */
+#	include <dev/ppbus/lpt.h>
+#endif
+#ifdef __unixware__
+#	include  <sys/filio.h>
 #endif
 
 extern int errno;
 
 #include "ParallelImp.h"
 
-
+#define LPRPort(foo) Java_gnu_io_LPRPort_ ## foo
+/* #define DEBUG */
 
 /*----------------------------------------------------------
 LPRPort.getOutputBufferFree
-   accept:    none  
-   perform:     
-   return:    number of bytes available in buffer.  
-   exceptions: none 
-   comments:  have not seen how to do this in the kernel yet.  
-----------------------------------------------------------*/ 
-JNIEXPORT jint JNICALL Java_gnu_io_LPRPort_getOutputBufferFree(JNIEnv *env,
-        jclass jclazz) {
+   accept:    none
+   perform:
+   return:    number of bytes available in buffer.
+   exceptions: none
+   comments:  have not seen how to do this in the kernel yet.
+----------------------------------------------------------*/
+JNIEXPORT jint JNICALL LPRPort(getOutputBufferFree)(JNIEnv *env,
+	jclass jclazz)
+{
 	printf("getOutputBufferFree is not implemented yet\n");
 
 	return(0);
 
 }
 /*----------------------------------------------------------
-/*----------------------------------------------------------
 LPRPort.setLPRMode
-   accept:     mode 
-   perform:    set the Printer communication mode 
+   accept:     mode
+   perform:    set the Printer communication mode
 	LPT_MODE_ANY:     pick the best possible mode
 	LPT_MODE_SPP:     compatibility mode/unidirectional
 	LPT_MODE_PS2:     byte mode/bidirectional
 	LPT_MODE_EPP:     extended parallel port
 	LPT_MODE_ECP:     enhanced capabilities port
-	LPT_MODE_NIBBLE:  Nibble Mode. Bi-directional. HP Bi-tronics. 
+	LPT_MODE_NIBBLE:  Nibble Mode. Bi-directional. HP Bi-tronics.
 	                  4 bits at a time.
-   return:     none 
+   return:     none
    exceptions: UnsupportedCommOperationException
-   comments:    
-----------------------------------------------------------*/ 
-JNIEXPORT jboolean JNICALL Java_gnu_io_LPRPort_setLPRMode(JNIEnv *env,
-        jclass jclazz, jint mode) {
-	switch(mode){
+   comments:
+----------------------------------------------------------*/
+JNIEXPORT jboolean JNICALL LPRPort(setLPRMode)(JNIEnv *env,
+	jclass jclazz, jint mode)
+{
+	switch(mode)
+	{
 		case LPT_MODE_ANY:
 			break;
 		case LPT_MODE_SPP:
@@ -111,41 +122,56 @@ JNIEXPORT jboolean JNICALL Java_gnu_io_LPRPort_setLPRMode(JNIEnv *env,
 		case LPT_MODE_ECP:
 		case LPT_MODE_NIBBLE:
 		default:
-			UnsupportedCommOperationException(env, "setLPRMode was unable to proced the requested mode" );
-
+			throw_java_exception( env, UNSUPPORTED_COMM_OPERATION,
+				"nativeSetSerialPortParams",
+				"setLPRMode was unable to proced the requested \
+				mode"
+			);
 	}
-
+	return(JNI_TRUE);
 }
 /*----------------------------------------------------------
 LPRPort.isPaperOut
-   accept:      none   
-   perform:     check if printer reports paper is out   
-   return:      Paper Out: JNI_TRUE  Not Paper Out: JNI_FALSE     
+   accept:      none
+   perform:     check if printer reports paper is out
+   return:      Paper Out: JNI_TRUE  Not Paper Out: JNI_FALSE
    exceptions:  none
    comments:    LP_NOPA    unchanged out-of-paper input, active high
-----------------------------------------------------------*/ 
-JNIEXPORT jboolean JNICALL Java_gnu_io_LPRPort_isPaperOut(JNIEnv *env,
-	jobject jobj){
+----------------------------------------------------------*/
+JNIEXPORT jboolean JNICALL LPRPort(isPaperOut)(JNIEnv *env,
+	jobject jobj)
+{
 
 	int status;
-	int fd = get_java_fd( env, jobj );
-
+	int fd = get_java_var( env, jobj,"fd","I" );
+#if defined (__linux__)
 	ioctl(fd, LPGETSTATUS,&status);
 	return( status & LP_NOPA ? JNI_TRUE : JNI_FALSE );
+#else
+/*  FIXME??  */
+	printf("ParallelImp.c LPGETSTATUS not defined\n");
+	return(JNI_TRUE);
+#endif
 }
 /*----------------------------------------------------------
 LPRPort.isPrinterBusy
-   accept:      none   
-   perform:     Check to see if the printer is printing.   
-   return:      JNI_TRUE if the printer is Busy, JNI_FALSE if its idle   
+   accept:      none
+   perform:     Check to see if the printer is printing.
+   return:      JNI_TRUE if the printer is Busy, JNI_FALSE if its idle
    exceptions:  none
    comments:    LP_BUSY     inverted busy input, active high
-----------------------------------------------------------*/ 
-JNIEXPORT jboolean JNICALL Java_gnu_io_LPRPort_isPrinterBusy(JNIEnv *env,
-	jobject jobj){
+----------------------------------------------------------*/
+JNIEXPORT jboolean JNICALL LPRPort(isPrinterBusy)(JNIEnv *env,
+	jobject jobj)
+{
 	int status;
-	int fd = get_java_fd( env, jobj );
+	int fd = get_java_var( env, jobj,"fd","I" );
+#if defined (__linux__)
 	ioctl(fd, LPGETSTATUS, &status);
+#else
+/*  FIXME??  */
+	printf("ParallelImp.c LPGETSTATUS not defined\n");
+#endif
 #if defined(__linux__)
 	return( status & LP_BUSY ? JNI_TRUE : JNI_FALSE );
 #endif
@@ -160,14 +186,21 @@ LPRPort.isPrinterError
    perform:     check for printer error
    return:      JNI_TRUE if there is an printer error otherwise JNI_FALSE
    exceptions:  none
-   comments:    LP_ERR   unchanged error input, active low 
-----------------------------------------------------------*/ 
-JNIEXPORT jboolean JNICALL Java_gnu_io_LPRPort_isPrinterError(JNIEnv *env,
-	jobject jobj){
+   comments:    LP_ERR   unchanged error input, active low
+----------------------------------------------------------*/
+JNIEXPORT jboolean JNICALL LPRPort(isPrinterError)(JNIEnv *env,
+	jobject jobj)
+{
 	int status;
-	int fd = get_java_fd( env, jobj );
+	int fd = get_java_var( env, jobj,"fd","I" );
+#if defined (__linux__)
 	ioctl(fd, LPGETSTATUS, &status);
 	return( status & LP_ERR ? JNI_TRUE : JNI_FALSE );
+#else
+/*  FIXME??  */
+	printf("ParallelImp.c LPGETSTATUS not defined\n");
+	return(JNI_FALSE);
+#endif
 }
 /*----------------------------------------------------------
 LPRPort.isPrinterSelected
@@ -175,14 +208,21 @@ LPRPort.isPrinterSelected
    perform:     check if printer is selected
    return:      JNI_TRUE if printer is selected other wise JNI_FALSE
    exceptions:  none
-   comments:    LP_SELEC   unchanged selected input, active high 
-----------------------------------------------------------*/ 
-JNIEXPORT jboolean JNICALL Java_gnu_io_LPRPort_isPrinterSelected(JNIEnv *env,
-	jobject jobj){
+   comments:    LP_SELEC   unchanged selected input, active high
+----------------------------------------------------------*/
+JNIEXPORT jboolean JNICALL LPRPort(isPrinterSelected)(JNIEnv *env,
+	jobject jobj)
+{
 	int status;
-	int fd = get_java_fd( env, jobj );
+	int fd = get_java_var( env, jobj,"fd","I" );
+#if defined (__linux__)
 	ioctl(fd, LPGETSTATUS, &status);
 	return( status & LP_SELEC ? JNI_TRUE : JNI_FALSE );
+#else
+/*  FIXME??  */
+	printf("ParallelImp.c LPGETSTATUS not defined\n");
+	return(JNI_FALSE);
+#endif
 }
 /*----------------------------------------------------------
 LPRPort.isPrinterTimedOut
@@ -193,17 +233,21 @@ LPRPort.isPrinterTimedOut
    exceptions:   none
    comments:     Is this documented right in the javadocs?
 	         not sure this is correct FIXME
-----------------------------------------------------------*/ 
-JNIEXPORT jboolean JNICALL Java_gnu_io_LPRPort_isPrinterTimedOut(JNIEnv *env,
-	jobject jobj){
+----------------------------------------------------------*/
+JNIEXPORT jboolean JNICALL LPRPort(isPrinterTimedOut)(JNIEnv *env,
+	jobject jobj)
+{
 	int status;
-	int fd = get_java_fd( env, jobj );
-	ioctl(fd, LPGETSTATUS, &status);
+	int fd = get_java_var( env, jobj,"fd","I" );
 #if defined(__linux__)
+	ioctl(fd, LPGETSTATUS, &status);
 	return( status & LP_BUSY ? JNI_TRUE : JNI_FALSE );
 #endif
 #if defined(__FreeBSD__)
+	printf("ParallelImp.c LPGETSTATUS not defined\n");
+	/*
 	return( status & EBUSY ? JNI_TRUE : JNI_FALSE );
+	*/
 #endif
 	return( JNI_FALSE );
 }
@@ -216,8 +260,8 @@ LPRPort.Initialize
    perform:     Initialize the native library
    return:      none
    comments:    lots of reading to do here. FIXME
-----------------------------------------------------------*/ 
-JNIEXPORT void JNICALL Java_gnu_io_LPRPort_Initialize( JNIEnv *env,
+----------------------------------------------------------*/
+JNIEXPORT void JNICALL LPRPort(Initialize)( JNIEnv *env,
 	jclass jclazz )
 {
 	/* This bit of code checks to see if there is a signal handler installed
@@ -236,15 +280,16 @@ LPRPort.open
    accept:      The device to open.  ie "/dev/lp0"
    perform:     open the device and return the filedescriptor
    return:      fd
-   exceptions:  IOExcepiton
+   exceptions:  PortInUseException
    comments:    Very often people complain about not being able to get past
-                this function and it turns out to be permissions on the 
+                this function and it turns out to be permissions on the
                 device file or bios has the device disabled.
-----------------------------------------------------------*/ 
-JNIEXPORT jint JNICALL Java_gnu_io_LPRPort_open( JNIEnv *env, jobject jobj,
+		FIXME  Lock Files?
+----------------------------------------------------------*/
+JNIEXPORT jint JNICALL LPRPort(open)( JNIEnv *env, jobject jobj,
 	jstring jstr )
 {
-	struct termios ttyset;
+	/*struct termios ttyset;*/
 	const char *filename = (*env)->GetStringUTFChars( env, jstr, 0 );
 	int fd = open( filename, O_RDWR | O_NONBLOCK );
 	(*env)->ReleaseStringUTFChars( env, jstr, NULL );
@@ -252,7 +297,8 @@ JNIEXPORT jint JNICALL Java_gnu_io_LPRPort_open( JNIEnv *env, jobject jobj,
 	return (jint)fd;
 
 fail:
-	IOException( env, strerror( errno ) );
+	throw_java_exception( env, PORT_IN_USE_EXCEPTION, "open",
+		strerror( errno ) );
 	return -1;
 }
 
@@ -264,11 +310,11 @@ LPRPort.nativeClose
    perform:     get the fd from the java end and close it
    return:      none
    exceptions:  none
-----------------------------------------------------------*/ 
-JNIEXPORT void JNICALL Java_gnu_io_LPRPort_nativeClose( JNIEnv *env,
+----------------------------------------------------------*/
+JNIEXPORT void JNICALL LPRPort(nativeClose)( JNIEnv *env,
 	jobject jobj )
 {
-	int fd = get_java_fd( env, jobj );
+	int fd = get_java_var( env, jobj,"fd","I" );
 
 	close( fd );
 	return;
@@ -281,39 +327,41 @@ LPRPort.writeByte
    perform:     write a single byte to the port
    return:      none
    exceptions:  IOException
-----------------------------------------------------------*/ 
-JNIEXPORT void JNICALL Java_gnu_io_LPRPort_writeByte( JNIEnv *env,
-	jobject jobj, jint ji ) 
+----------------------------------------------------------*/
+JNIEXPORT void JNICALL LPRPort(writeByte)( JNIEnv *env,
+	jobject jobj, jint ji )
 {
 	unsigned char byte = (unsigned char)ji;
-	int fd = get_java_fd( env, jobj );
+	int fd = get_java_var( env, jobj,"fd","I" );
 
 	if( write( fd, &byte, sizeof( unsigned char ) ) >= 0 ) return;
-	IOException( env, strerror( errno ) );
+	throw_java_exception( env, IO_EXCEPTION, "writeByte",
+		strerror( errno ) );
 }
 
 
 /*----------------------------------------------------------
 LPRPort.writeArray
 
-   accept:      jbarray: bytes used for writing 
+   accept:      jbarray: bytes used for writing
                 offset: offset in array to start writing
                 count: Number of bytes to write
    perform:     write length bytes of jbarray
    return:      none
    exceptions:  IOException
-----------------------------------------------------------*/ 
-JNIEXPORT void JNICALL Java_gnu_io_LPRPort_writeArray( JNIEnv *env,
+----------------------------------------------------------*/
+JNIEXPORT void JNICALL LPRPort(writeArray)( JNIEnv *env,
 	jobject jobj, jbyteArray jbarray, jint offset, jint count )
 {
-	int fd = get_java_fd( env, jobj );
+	int fd = get_java_var( env, jobj,"fd","I" );
 	jbyte *body = (*env)->GetByteArrayElements( env, jbarray, 0 );
 	unsigned char *bytes = (unsigned char *)malloc( count );
 	int i;
 	for( i = 0; i < count; i++ ) bytes[ i ] = body[ i + offset ];
 	(*env)->ReleaseByteArrayElements( env, jbarray, body, 0 );
 	if( write( fd, bytes, count ) < 0 )
-		IOException( env, strerror( errno ) );
+		throw_java_exception( env, IO_EXCEPTION, "writeArray",
+			strerror( errno ) );
 	free( bytes );
 }
 
@@ -333,7 +381,7 @@ read_byte_array
                 >0 number of bytes read
    comments:    According to the Communications API spec, a receive threshold
                 of 1 is the same as having the threshold disabled.
-----------------------------------------------------------*/ 
+----------------------------------------------------------*/
 int read_byte_array( int fd, unsigned char *buffer, int length, int threshold,
 	int timeout )
 {
@@ -347,8 +395,10 @@ int read_byte_array( int fd, unsigned char *buffer, int length, int threshold,
 	sleep.tv_usec = 1000 * ( timeout % 1000 );
 	left = length;
 
-	while( bytes < length && bytes < threshold ) {
-		if( timeout > 0 ) {
+	while( bytes < length && bytes < threshold )
+	{
+		if( timeout > 0 )
+		{
          /* FIXME: In Linux, select updates the timeout automatically, so
             other OSes will need to update it manually if they want to have
             the same behavior.  For those OSes, timeouts will occur after no
@@ -376,22 +426,22 @@ LPRPort.readByte
    perform:     Read a single byte from the port
    return:      The byte read
    exceptions:  IOException
-----------------------------------------------------------*/ 
-JNIEXPORT jint JNICALL Java_gnu_io_LPRPort_readByte( JNIEnv *env,
+----------------------------------------------------------*/
+JNIEXPORT jint JNICALL LPRPort(readByte)( JNIEnv *env,
 	jobject jobj )
-{ 
+{
 	int bytes, fd, timeout;
 	unsigned char buffer[ 1 ];
-	jfieldID jfield;
-	jclass jclazz = (*env)->GetObjectClass( env, jobj );
-	jfield = (*env)->GetFieldID( env, jclazz, "fd", "I" );
-	fd = (int)( (*env)->GetIntField( env, jobj, jfield ) );
-	jfield = (*env)->GetFieldID( env, jclazz, "timeout", "I" );
-	timeout = (int)( (*env)->GetIntField( env, jobj, jfield ) );
+
+	fd = get_java_var( env, jobj,"fd","I" );
+	timeout = get_java_var( env, jobj, "timeout","I");
 
 	bytes = read_byte_array( fd, buffer, 1, 1, timeout );
-	if( bytes < 0 ) {
-		IOException( env, strerror( errno ) );
+	if( bytes < 0 )
+	{
+		throw_java_exception( env, IO_EXCEPTION, "readByte",
+			strerror( errno ) );
+
 		return -1;
 	}
 	return (bytes ? (jint)buffer[ 0 ] : -1);
@@ -407,37 +457,40 @@ LPRPort.readArray
                  0 on read timeout
    exceptions:   IOException
    comments:     throws IOException if asked to read > SSIZE_MAX
-----------------------------------------------------------*/ 
-JNIEXPORT jint JNICALL Java_gnu_io_LPRPort_readArray( JNIEnv *env,
+----------------------------------------------------------*/
+JNIEXPORT jint JNICALL LPRPort(readArray)( JNIEnv *env,
 	jobject jobj, jbyteArray jbarray, jint offset, jint length )
-{  
+{
 	int bytes, i, fd, threshold, timeout;
 	jbyte *body;
 	unsigned char *buffer;
-	jfieldID jfield;
-	jclass jclazz = (*env)->GetObjectClass( env, jobj );
-	jfield = (*env)->GetFieldID( env, jclazz, "fd", "I" );
-	fd = (int)( (*env)->GetIntField( env, jobj, jfield ) );
-	jfield = (*env)->GetFieldID( env, jclazz, "threshold", "I" );
-	threshold = (int)( (*env)->GetIntField( env, jobj, jfield ) );
-	jfield = (*env)->GetFieldID( env, jclazz, "timeout", "I" );
-	timeout = (int)( (*env)->GetIntField( env, jobj, jfield ) );
+	fd = get_java_var( env, jobj,"fd","I" );
+	threshold = get_java_var( env, jobj,"threshold","I" );
+	timeout = get_java_var( env, jobj,"threshold","I" );
 
-	if( length < 1 || length > SSIZE_MAX ) {
-		IOException( env, "Invalid length" );
+	if( length < 1 || length > SSIZE_MAX )
+	{
+		throw_java_exception( env, IO_EXCEPTION, "readArray",
+			"Invalid length" );
 		return -1;
 	}
 
 	buffer = (unsigned char *)malloc( sizeof( unsigned char ) * length );
-	if( buffer == 0 ) {
-		IOException( env, "Unable to allocate buffer" );
+	if( buffer == 0 )
+	{
+		throw_java_exception( env, IO_EXCEPTION, "writeByte",
+			"Unable to allocate buffer" );
+
 		return -1;
 	}
 
 	bytes = read_byte_array( fd, buffer, length, threshold, timeout );
-	if( bytes < 0 ) {
+	if( bytes < 0 )
+	{
 		free( buffer );
-		IOException( env, strerror( errno ) );
+		throw_java_exception( env, IO_EXCEPTION, "readArray",
+			strerror( errno ) );
+
 		return -1;
 	}
 
@@ -457,15 +510,54 @@ LPRPort.nativeavailable
    return:      available bytes
                 -1 on error
    exceptions:  none
-----------------------------------------------------------*/ 
-JNIEXPORT jint JNICALL Java_gnu_io_LPRPort_nativeavailable( JNIEnv *env,
+----------------------------------------------------------*/
+JNIEXPORT jint JNICALL LPRPort(nativeavailable)( JNIEnv *env,
 	jobject jobj )
 {
-	int fd = get_java_fd( env, jobj );
+	int fd = get_java_var( env, jobj,"fd","I" );
 	int result;
+/*
+	char message[80];
 
-	if( ioctl( fd, FIONREAD, &result ) ) return -1;
-	else return (jint)result;
+	ENTER( "LPRPort:nativeavailable" );
+*/
+/*
+    On SCO OpenServer FIONREAD always fails for serial devices,
+    so try ioctl FIORDCHK instead; will only tell us whether
+    bytes are available, not how many, but better than nothing.
+
+    This turns out to be true on Solaris also.  taj.
+*/
+#ifdef FIORDCHK  /* __unixware__ __sun__ probably others */
+	result = ioctl(fd, FIORDCHK, 0);
+#else
+	if( ioctl( fd, FIONREAD, &result ) < 0 )
+	{
+		goto fail;
+	}
+#endif /* FIORDCHK */
+	if (result == -1) {
+		goto fail;
+	}
+	if( result )
+	{
+/*		sprintf(message, "    nativeavailable: FIORDCHK result %d, \
+				errno %d\n", result , result == -1 ? errno : 0);
+		report( message );
+*/
+	}
+/*
+	LEAVE( "LPRPort:nativeavailable" );
+*/
+	return (jint)result;
+fail:
+/*
+	report("LPRPort:nativeavailable:  ioctl() failed\n");
+	LEAVE( "LPRPort:nativeavailable" );
+*/
+	throw_java_exception( env, IO_EXCEPTION, "nativeavailable",
+		strerror( errno ) );
+	return (jint)result;
 }
 
 
@@ -477,14 +569,13 @@ LPRPort.setHWFC
    return:      none
    exceptions:  IOException
 ----------------------------------------------------------*/
-JNIEXPORT void JNICALL Java_gnu_io_LPRPort_setHWFC( JNIEnv *env,
+JNIEXPORT void JNICALL LPRPort(setHWFC)( JNIEnv *env,
 	jobject jobj, jboolean state )
 {
-	int fd = get_java_fd( env, jobj );
+	/* int fd = get_java_var( env, jobj,"fd","I" ); */
 	return;
 }
 
-
 /*----------------------------------------------------------
 LPRPort.eventLoop
 
@@ -492,160 +583,259 @@ LPRPort.eventLoop
    perform:     periodically check for ParallelPortEvents
    return:      none
    exceptions:  none
-   comments:    lots of work needed here
-struct lp_stats {
-        unsigned long chars;
-        unsigned long sleeps;
-        unsigned int maxrun;
-        unsigned int maxwait;
-        unsigned int meanwait;
-        unsigned int mdev;
+   comments:    lots of work needed here.  Yes its a mess.
+struct lp_stats
+{
+	unsigned long chars;
+	unsigned long sleeps;
+	unsigned int maxrun;
+	unsigned int maxwait;
+	unsigned int meanwait;
+	unsigned int mdev;
 };
 
-----------------------------------------------------------*/ 
-JNIEXPORT void JNICALL Java_gnu_io_LPRPort_eventLoop( JNIEnv *env,
+----------------------------------------------------------*/
+JNIEXPORT void JNICALL LPRPort(eventLoop)( JNIEnv *env,
 	jobject jobj )
 {
-	int fd, ret, change;
+	int fd, ret;
 	unsigned int pflags;
 	fd_set rfds;
 	struct timeval sleep;
-	jfieldID jfield;
-	jmethodID method, interrupt;
 	jboolean interrupted = 0;
-	jclass jclazz, jthread;
-	jclazz = (*env)->GetObjectClass( env, jobj );
-	jfield = (*env)->GetFieldID( env, jclazz, "fd", "I" );
-	fd = (int)( (*env)->GetIntField( env, jobj, jfield ) );
-	method = (*env)->GetMethodID( env, jclazz, "sendEvent", "(IZ)V" );
-	jthread = (*env)->FindClass( env, "java/lang/Thread" );
-	interrupt = (*env)->GetStaticMethodID( env, jthread, "interrupted", "()Z" );
+
+	fd = get_java_var( env, jobj,"fd","I" );
+
+	interrupted = is_interrupted(env, jobj);
 
 	FD_ZERO( &rfds );
-	while( !interrupted ) {
+	while( !interrupted )
+	{
 		FD_SET( fd, &rfds );
-		sleep.tv_sec = 1;	/* Check every 1 second, or on receive data */
+		/* Check every 1 second, or on receive data */
+		sleep.tv_sec = 1;	
 		sleep.tv_usec = 0;
-		do { 
+		do {
 			ret = select( fd + 1, &rfds, NULL, NULL, &sleep );
 		}
 		while (ret < 0 && errno == EINTR);
 		if( ret < 0 ) break;
-		if( ioctl( fd, LPGETSTATUS, &pflags ) ) break;
+		interrupted = is_interrupted(env, jobj);
+		if(interrupted) {
+			return;
+		}
+
+#if defined(LPGETSTATUS)
+		ioctl( fd, LPGETSTATUS, &pflags );
+#else
+	/*  FIXME??  */
+	printf("ParallelImp.c LPGETSTATUS is undefined!\n");
+#endif
 
 /*
-                       PAR_EV_BUFFER:
-                       PAR_EV_ERROR:
+			PAR_EV_BUFFER:
+			PAR_EV_ERROR:
 */
 
-#if defined(__linux__)
+#if defined(LP_BUSY)
 		if (pflags&LP_BUSY)    /* inverted input, active high */
-#endif
-#if defined(__FreeBSD__)
+			send_event( env, jobj, PAR_EV_ERROR, JNI_TRUE );
+#elif defined(EBUSY)
 		if (pflags&EBUSY)    /* inverted input, active high */
-#endif
-			(*env)->CallVoidMethod( env, jobj, method, (jint)PAR_EV_ERROR, JNI_TRUE );
+			send_event( env, jobj, PAR_EV_ERROR, JNI_TRUE );
+#endif /* EBUSY LP_BUSY */
 /*  FIXME  this has moved into the ifdef __kernel__?  Need to get the
            posix documentation on this.
-		if (pflags&LP_ACK)   
-			(*env)->CallVoidMethod( env, jobj, method, (jint)PAR_EV_ERROR, JNI_TRUE );
-*/  /* unchanged input, active low */
+		if (pflags&LP_ACK)
+			send_event( env, jobj, PAR_EV_ERROR, JNI_TRUE );
+*/
+#if defined (__linux__)
+		/* unchanged input, active low */
 		if (pflags&LP_NOPA)   /* unchanged input, active high */
-			(*env)->CallVoidMethod( env, jobj, method, (jint)PAR_EV_ERROR, JNI_TRUE );
+			send_event( env, jobj, PAR_EV_ERROR, 1 );
 		if (pflags&LP_SELEC)  /* unchanged input, active high */
-			(*env)->CallVoidMethod( env, jobj, method, (jint)PAR_EV_ERROR, JNI_TRUE );
+			send_event( env, jobj, PAR_EV_ERROR, 1 );
 		if (pflags&LP_ERR)  /* unchanged input, active low */
-			(*env)->CallVoidMethod( env, jobj, method, (jint)PAR_EV_ERROR, JNI_TRUE );
-		interrupted = (*env)->CallStaticBooleanMethod( env, jthread, interrupt );
+			send_event( env, jobj, PAR_EV_ERROR, 1 );
+#else
+	/*  FIXME??  */
+	printf("ParallelImp.c LPGETSTATUS is undefined!\n");
+#endif
+		usleep(1000);
 	}
 	return;
 }
 
-
-/*----------------------------------------------------------
- send_printer_events
-
-   accept:      int    event     ParallelPortEvent constant
-                int    change    Number of times this event happened
-                int    state     current state: 0 is false, nonzero is true
-   perform:     Send the necessary events
-   return:      none
-   exceptions:  none
-   comments:    Since the interrupt counters tell us how many times the
-                state has changed, we can send a ParallelPortEvent for each
-                interrupt (change) that has occured.  If we don't do this,
-                we'll miss a whole bunch of events.
-----------------------------------------------------------*/ 
-void send_printer_events( JNIEnv *env, jobject jobj, jmethodID method,
-	int event, int change, int state )
+JNIEXPORT void JNICALL LPRPort(setInputBufferSize)(JNIEnv *env,
+	jobject jobj, jint size )
 {
-	int i, s;
-	jboolean flag;
-	if( state ) s = 1;
-	else s = 0;
-
-	for( i = 0; i < change; i++ ) {
-		if( ( change + s + i ) % 2 ) flag = JNI_FALSE;
-		else flag = JNI_TRUE;
-		(*env)->CallVoidMethod( env, jobj, method, (jint)event, flag );
-	}
+#ifdef DEBUG
+	printf("setInputBufferSize is not implemented\n");
+#endif
+}
+JNIEXPORT jint JNICALL LPRPort(getInputBufferSize)(JNIEnv *env,
+	jobject jobj)
+{
+#ifdef DEBUG
+	printf("getInputBufferSize is not implemented\n");
+#endif
+	return(1);
+}
+JNIEXPORT void JNICALL LPRPort(setOutputBufferSize)(JNIEnv *env,
+	jobject jobj, jint size )
+{
+#ifdef DEBUG
+	printf("setOutputBufferSize is not implemented\n");
+#endif
+}
+JNIEXPORT jint JNICALL LPRPort(getOutputBufferSize)(JNIEnv *env,
+	jobject jobj)
+{
+#ifdef DEBUG
+	printf("getOutputBufferSize is not implemented\n");
+#endif
+	return(1);
 }
-
-
 /*----------------------------------------------------------
-get_java_fd
+throw_java_exception
 
    accept:      env (keyhole to java)
-                jobj (java NativePort object)
+                *exc (exception class name)
+                *foo (function name)
+                *msg (error message)
+   perform:     Throw a new java exception
+   return:      none
+   exceptions:  haha!
+   comments:
+----------------------------------------------------------*/
+void throw_java_exception( JNIEnv *env, char *exc, char *foo, char *msg )
+{
+	char buf[ 60 ];
+	jclass clazz = (*env)->FindClass( env, exc );
+	if( !clazz )
+	{
+		(*env)->ExceptionDescribe( env );
+		(*env)->ExceptionClear( env );
+		return;
+	}
+#if defined(_GNU_SOURCE)
+	snprintf( buf, 60, "%s in %s", msg, foo );
+#else
+	sprintf( buf,"%s in %s", msg, foo );
+#endif /* _GNU_SOURCE */
+	(*env)->ThrowNew( env, clazz, buf );
+/* ct7 * Added DeleteLocalRef */
+	(*env)->DeleteLocalRef( env, clazz );
+}
+
+/*----------------------------------------------------------
+ report
+
+   accept:      string to send to stderr     
+   perform:     if DEBUG is defined send the string to stderr.
+   return:      none
+   exceptions:  none
+   comments:
+----------------------------------------------------------*/
+void report(char *msg)
+{
+#ifdef DEBUG
+        fprintf(stderr, msg);
+#endif /* DEBUG */
+}
+
+/*----------------------------------------------------------
+ is_interrupted
+
+   accept:      
+   perform:     see if the port is being closed. 
+   return:      a positive value if the port is being closed.
+   exceptions:  none
+   comments:
+----------------------------------------------------------*/
+jboolean is_interrupted(JNIEnv *env, jobject jobj)
+{
+	jmethodID foo;
+	jclass jclazz;
+	int result;
+
+	(*env)->ExceptionClear(env);
+
+	jclazz = (*env)->GetObjectClass( env, jobj );
+	if(jclazz == NULL) return JNI_TRUE;
+
+	foo = (*env)->GetMethodID( env, jclazz, "checkMonitorThread", "()Z");
+	if(foo == NULL) return JNI_TRUE;
+
+	result = (*env)->CallBooleanMethod( env, jobj, foo );
+
+#ifdef DEBUG
+	if((*env)->ExceptionOccurred(env)) {
+		report ("an error occured calling sendEvent()\n");
+		(*env)->ExceptionDescribe(env);
+		(*env)->ExceptionClear(env);
+	}
+#endif /* DEBUG */
+	(*env)->DeleteLocalRef( env, jclazz );
+
+	return(result);
+}
+/*----------------------------------------------------------
+ send_event
+
+   accept:      The event type and the event state     
+   perform:     if state is > 0 send a JNI_TRUE event otherwise send JNI_FALSE
+   return:      a positive value if the port is being closed.
+   exceptions:  none
+   comments:
+----------------------------------------------------------*/
+int send_event(JNIEnv *env, jobject jobj, jint type, int flag)
+{
+	int result;
+	jmethodID foo;
+	jclass jclazz = (*env)->GetObjectClass( env, jobj );
+
+	if(jclazz == NULL) return JNI_TRUE;
+	foo = (*env)->GetMethodID( env, jclazz, "sendEvent", "(IZ)Z" );
+
+	(*env)->ExceptionClear(env);
+
+	result = (*env)->CallBooleanMethod( env, jobj, foo, type, 
+		flag > 0 ? JNI_TRUE : JNI_FALSE );
+
+#ifdef DEBUG
+	if((*env)->ExceptionOccurred(env)) {
+		report ("an error occured calling sendEvent()\n");
+		(*env)->ExceptionDescribe(env);
+		(*env)->ExceptionClear(env);
+	}
+#endif /* DEBUG */
+	(*env)->DeleteLocalRef( env, jclazz );
+	return(result);
+}
+/*----------------------------------------------------------
+get_java_var
+
+   accept:      env (keyhole to java)
+                jobj (java RXTXPort object)
+
    return:      the fd field from the java object
    exceptions:  none
    comments:
-----------------------------------------------------------*/ 
-int get_java_fd( JNIEnv *env, jobject jobj )
+----------------------------------------------------------*/
+int get_java_var( JNIEnv *env, jobject jobj, char *id, char *type )
 {
+	int result = 0;
 	jclass jclazz = (*env)->GetObjectClass( env, jobj );
-	jfieldID jfd = (*env)->GetFieldID( env, jclazz, "fd", "I" );
-	if( !jfd ) {
+	jfieldID jfd = (*env)->GetFieldID( env, jclazz, id, type );
+	if( !jfd )
+	{
 		(*env)->ExceptionDescribe( env );
 		(*env)->ExceptionClear( env );
-		return 0;
+		return result;
 	}
-	return (int)( (*env)->GetIntField( env, jobj, jfd ) );
-}
-
-
-/*----------------------------------------------------------
-IOException
-
-   accept:      env (keyhole to java)
-                *msg (error message)
-   perform:     Throw a java.io.IOException
-   return:      none
-   exceptions:  haha!
-   comments:
-----------------------------------------------------------*/ 
-void IOException( JNIEnv *env, char *msg )
-{
-	jclass clazz = (*env)->FindClass( env, "java/io/IOException" );
-	if( clazz == 0 ) return;
-	(*env)->ThrowNew( env, clazz, msg );
-}
-
-
-/*----------------------------------------------------------
-UnsupportedCommOperationException
-
-   accept:      env (keyhole to java)
-                *msg (error message)
-   perform:     Throw a javax.comm.UnsupportedCommOperationException
-   return:      none
-   exceptions:  haha!
-   comments:
-----------------------------------------------------------*/ 
-void UnsupportedCommOperationException( JNIEnv *env, char *msg )
-{
-	jclass clazz = (*env)->FindClass( env,
-		"javax/comm/UnsupportedCommOperationException" );
-	if( clazz == 0 ) return;
-	(*env)->ThrowNew( env, clazz, msg );
+	result = (int)( (*env)->GetIntField( env, jobj, jfd ) );
+/* ct7 & gel * Added DeleteLocalRef */
+	(*env)->DeleteLocalRef( env, jclazz );
+	return result;
 }
