@@ -1,6 +1,6 @@
 /*-------------------------------------------------------------------------
 |   rxtx is a native interface to serial ports in java.
-|   Copyright 1997, 1998 by Trent Jarvi jarvi@ezlink.com.
+|   Copyright 1997-2002 by Trent Jarvi taj@www.linux.org.uk.
 |
 |   This library is free software; you can redistribute it and/or
 |   modify it under the terms of the GNU Library General Public
@@ -24,12 +24,47 @@
 #define _WIN32S_H_
 #include <windows.h>
 #include <sys/types.h>
-//#include "jni.h"
 #include <io.h>
+#ifdef TRACE
+#define ENTER(x) report("entering "x" \n");
+#define LEAVE(x) report("leaving "x" \n");
+#else
+#define ENTER(x)
+#define LEAVE(x)
+#endif /* TRACE */
+#define YACK() \
+{ \
+	char *allocTextBuf, message[80]; \
+	unsigned long nChars; \
+	unsigned int errorCode = GetLastError(); \
+	nChars = FormatMessage ( \
+		FORMAT_MESSAGE_ALLOCATE_BUFFER | \
+		FORMAT_MESSAGE_FROM_SYSTEM, \
+		NULL, \
+		errorCode, \
+		MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), \
+		(LPSTR)&allocTextBuf, \
+		16, \
+		NULL ); \
+	sprintf( message, "Error 0x%x at %s(%d): %s\n", errorCode, __FILE__, __LINE__, allocTextBuf); \
+	report_error( message ); \
+	LocalFree(allocTextBuf); \
+}
+
 typedef unsigned char   cc_t;
 typedef unsigned int    speed_t;
 typedef unsigned int    tcflag_t;
 
+/* structs are from linux includes or linux man pages to match
+   interfaces.
+*/
+
+struct timespec
+{
+	time_t	tv_sec;
+	long	tv_nsec;
+};
+
 #define NCCS 32
 struct termios
   {
@@ -43,14 +78,63 @@ struct termios
     speed_t c_ospeed;           /* output speed */
   };
 
-int serial_open(const char *File, int flags);
+/*  for TIOCGSERIAL and TIOCSSERIAL of interest are baud_base and
+ *  custom_divisor
+ *  --- NOTE:  This is not used.  Win32 sets custom speeds on the
+ *             kernel side.
+ */
+struct serial_struct {
+/*
+	Mainly we are after baud_base/custom_diviser to match
+	the ioctl() in SerialImp.c
+*/
+	int custom_divisor;   /* use to set unsupported speeds */
+	int baud_base;        /* use to set unsupported speeds */
+
+	unsigned short	close_delay, closing_wait, iomem_reg_shift;
+	int type, line, irq, flags, xmit_fifo_size, hub6;
+	unsigned int	port, port_high;
+	char		io_type;
+	unsigned char	*iomem_base;
+};
+struct serial_icounter_struct {
+	int cts;		/* clear to send count */
+	int dsr;		/* data set ready count */
+	int rng;		/* ring count */
+	int dcd;		/* carrier detect count */
+	int rx;			/* recieved byte count */
+	int tx;			/* transmitted byte count */
+	int frame;		/* frame error count */
+	int overrun;		/* hardware overrun error count */
+	int parity;		/* parity error count */
+	int brk;		/* break count */
+	int buf_overrun;	/* buffer overrun count */
+	int reserved[9]; 	/* unused */
+};
+
+int serial_test( char * );
+int serial_open(const char *File, int flags, ... );
+int serial_close(int fd);
 int serial_read(int fd, void *b, int size);
 int serial_write(int fd, char *Str, int length);
-#define open serial_open
-#define read serial_read
-#define write serial_write
+/*
+ * lcc winsock.h conflicts
+ */
+#ifndef __LCC__
+int serial_select(int, struct fd_set *, struct fd_set *, struct fd_set *, struct timeval *);
+#define select serial_select
+#endif
 
-int fcntl(int fd, int command, int arg);
+#define OPEN serial_open
+#define CLOSE serial_close
+#define READ serial_read
+#define WRITE serial_write
+
+void termios_interrupt_event_loop( int , int );
+void termios_setflags( int , int[] );
+struct termios_list *find_port( int );
+void usleep(unsigned long usec);
+int fcntl(int fd, int command, ...);
 const char *get_dos_port(const char *);
 void set_errno(int);
 char *sterror(int);
@@ -60,7 +144,7 @@ int termios_to_bytesize(int);
 int bytesize_to_termios(int);
 int tcgetattr(int Fd, struct termios *s_termios);
 int tcsetattr(int Fd, int when, struct termios *);
-int close(int );
+int serial_close(int );
 speed_t cfgetospeed(struct termios *s_termios);
 speed_t cfgetispeed(struct termios *s_termios);
 int cfsetspeed(struct termios *, speed_t speed);
@@ -72,8 +156,13 @@ int tcsetpgrp ( int , int );
 int tcdrain ( int );
 int tcflow ( int , int );
 int tcsendbreak ( int , int );
-int ioctl(int fd, int request, unsigned int *arg);
+int ioctl(int fd, int request, ... );
+/*
+int fstat(int fd, ... );
+*/
 void cfmakeraw(struct termios *s_termios);
+int termiosGetParityErrorChar( int );
+void termiosSetParityError( int, char );
 
 #define O_NOCTTY	0400	/* not for fcntl */
 #define O_NONBLOCK	 00004
@@ -206,21 +295,7 @@ void cfmakeraw(struct termios *s_termios);
 #define  B9600	0000015
 #define  B19200	0000016
 #define  B38400	0000017
-#define EXTA B19200
-#define EXTB B38400
-#define CSIZE	0000060
-#define   CS5	0000000
-#define   CS6	0000020
-#define   CS7	0000040
-#define   CS8	0000060
-#define CSTOPB	0000100
-#define CREAD	0000200
-#define PARENB	0000400
-#define PARODD	0001000
-#define HUPCL	0002000
-#define CLOCAL	0004000
-# define CBAUDEX 0010000
-#define  B57600   0010001
+#define  B57600	  0010001
 #define  B115200  0010002
 #define  B230400  0010003
 #define  B460800  0010004
@@ -235,25 +310,65 @@ void cfmakeraw(struct termios *s_termios);
 #define  B3000000 0010015
 #define  B3500000 0010016
 #define  B4000000 0010017
+
+/* glue for unsupported linux speeds see also SerialImp.h.h */
+
+#define B14400		1010001
+#define B28800		1010002
+#define B128000		1010003
+#define B256000		1010004
+
+#define EXTA B19200
+#define EXTB B38400
+#define CSIZE	0000060
+#define   CS5	0000000
+#define   CS6	0000020
+#define   CS7	0000040
+#define   CS8	0000060
+#define CSTOPB	0000100
+#define CREAD	0000200
+#define PARENB	0000400
+#define PARODD	0001000
+#define HUPCL	0002000
+#define CLOCAL	0004000
+# define CBAUDEX 0010000
 # define CIBAUD	  002003600000		/* input baud rate (not used) */
 # define CRTSCTS  020000000000		/* flow control */
 
-/* c_lflag bits */
-#define ISIG	0x00000080
-#define ICANON	0x00000100
-#define XCASE	0x00004000
-#define ECHO	0x00000008
-#define ECHOE	0x00000002
-#define ECHOK	0x00000004
-#define ECHONL	0x00000010
-#define NOFLSH	0x80000000
-#define TOSTOP	0x00400000
-#define ECHOCTL	0x00000040
-#define ECHOPRT	0x00000020
-#define ECHOKE	0x00000001
-#define FLUSHO	0x00800000
-#define PENDIN	0x20000000
-#define IEXTEN	0x00000400
+/* c_l flag */
+#define ISIG    0000001
+#define ICANON  0000002
+#define XCASE   0000004
+#define ECHO    0000010
+#define ECHOE   0000020
+#define ECHOK   0000040
+#define ECHONL  0000100
+#define NOFLSH  0000200
+#define TOSTOP  0000400
+#define ECHOCTL 0001000
+#define ECHOPRT 0002000
+#define ECHOKE  0004000
+#define FLUSHO  0010000
+#define PENDIN  0040000
+#define IEXTEN  0100000
+
+/* glue for unsupported windows speeds */
+
+#define CBR_230400	230400
+#define CBR_28800	28800
+#define CBR_460800	460800
+#define CBR_500000	500000
+#define CBR_576000	576000
+#define CBR_921600	921600
+#define CBR_1000000	1000000
+#define CBR_1152000	1152000
+#define CBR_1500000	1500000
+#define CBR_2000000	2000000
+#define CBR_2500000	2500000
+#define CBR_3000000	3000000
+#define CBR_3500000	3500000
+#define CBR_4000000	4000000
+
 
 /* Values for the ACTION argument to `tcflow'.  */
 #define	TCOOFF		0
@@ -270,11 +385,47 @@ void cfmakeraw(struct termios *s_termios);
 #define	TCSANOW		0
 #define	TCSADRAIN	1
 #define	TCSAFLUSH	2
-#endif /*_WIN32S_H_*/
 
 /* ioctls */
+#define TIOCSERGETLSR	0x5459
+
+#endif /*_WIN32S_H_*/
+
+/* unused ioctls */
+#define TCSBRK		0x5409
+#define TIOCOUTQ	0x5411
 #define TIOCMGET	0x5415
+#define TIOCMBIS	0x5416
+#define TIOCMBIC	0x5417
 #define TIOCMSET	0x5418
+#define TIOCGSOFTCAR	0x5419
+#define TIOCSSOFTCAR	0x541a
+#define TIOCSER_TEMP	0x01
+/*
+#define FIONREAD	0x541b
+TIOC[GS]SERIAL is not used on win32.  It was dropped after we could not
+find a way to get/set buad_base and divisor directly.
+#define TIOCGSERIAL	0x541e
+#define TIOCSSERIAL	0x541f
+*/
+#define TCSBRKP		0x5425
+#define TIOCSERCONFIG	0x5453
+#define TIOCSERGWILD	0x5454
+#define TIOCSERSWILD	0x5455
+#define TIOCSERGSTRUCT	0x5458
+#define TIOCSERGETMULTI	0x545a
+#define TIOCSERSETMULTI	0x545b
+#define TIOCMIWAIT	0x545c
+/* this would require being able to get the number of overruns ... */
+/*
+	FIXME
+	frame and parity errors caused crashes in testing BlackBox
+	#define TIOCGICOUNT	0x545d
+*/
+
+/* ioctl errors */
+#define ENOIOCTLCMD	515
+#define EBADFD		 77
 /* modem lines */
 #define TIOCM_LE    0x001
 #define TIOCM_DTR   0x002
